% Function to compare Segmentation results 
% 
% created by Andre Marcal,  4 Jan. 2019
%         last modified on  4 Jan. 2019
%
% Citations:
% Marcal, A.R.S. (2019). 
% Segmentation of Sentinel-2 images on SNAP - An evaluation with SITEF. 
% IGARSS 2019 (in press).
% AND / OR
% Marcal, A.R.S., Rodrigues, A.S. (2009). 
% A method for multi-spectral image segmentation evaluation based on synthetic images, 
% Computers & Geosciences, Vol.35, No.8, 1574-1581.
%
% INPUTS
% S1 - Segmentation Image 1 
% S2 - Segmentation Image 2  
%  
% OUTPUTS
% R  - Rand index
% CR - Corrected Rand index
% J  - Jaccard index
%
% e.g. [R,CR,J] = RandJaccard(IS1,IS2)

function [R,CR,J] = RandJaccard(S1,S2)
    [NL,NC,NB]=size(S1);
    [NL2,NC2,NB2]=size(S2);
    R=99; CR=99; J=99;
    if (NL==NL2) && (NC==NC2) && (NB==1) && (NB2==1)
        S1v=reshape(S1,1,NL*NC);
        S2v=reshape(S2,1,NL*NC);
        NoE1=max(max(S1v));
        NoE2=max(max(S2v));
        if (NoE1<100000) && (NoE2<100000)
            nbins = [NoE1 NoE2];
            [Hist2D,Xedges,Yedges] = histcounts2(S1v,S2v,nbins);
            MATsum=sum(sum(Hist2D));
            MATsum2=sum(sum(Hist2D.*Hist2D));
            COLsum2=sum(sum(Hist2D).*sum(Hist2D));
            LINsum2=sum(sum(Hist2D').*sum(Hist2D'));
            a=sum(sum(Hist2D.*(Hist2D-1)))/2;  % (i) in Hubert & Arabie
            d=(MATsum^2+MATsum2-COLsum2-LINsum2)/2;  % (ii) 
            c=(COLsum2-MATsum2)/2;                   % (iii)
            b=(LINsum2-MATsum2)/2;                   % (iv) 
            %controlo=[a b c d]
            R=(a+d)/(a+b+c+d);
            J=a/(a+b+c);
            COLcomb=sum(sum(Hist2D).*(sum(Hist2D)-1))/2;
            LINcomb=sum(sum(Hist2D').*(sum(Hist2D')-1))/2;
            MATcomb=MATsum*(MATsum-1)/2;
            Rexp=1+2*COLcomb*LINcomb/(MATcomb^2)-(COLcomb+LINcomb)/MATcomb;
            CR=max(0,(R-Rexp)/(1-Rexp));
        end
    end

end