% Script to create Synthetic Images (SITEF 2019)
%  (only Base and Label images)
% 
% created by Andre Marcal,  2 Jan. 2019
%         last modified on  2 Jan. 2019
%
% Citations:
% Marcal, A.R.S. (2019). 
% Segmentation of Sentinel-2 images on SNAP - An evaluation with SITEF. 
% IGARSS 2019 (in press).
% AND / OR
% Marcal, A.R.S., Rodrigues, A.S. (2009). 
% A method for multi-spectral image segmentation evaluation based on synthetic images, 
% Computers & Geosciences, Vol.35, No.8, 1574-1581.
%
% INPUTS
% u - unit size (pixels) 
% s - maximum size (units) 
% r - repetition factor 
% t - No of class types (min 3)
%
% e.g. [BaseImage,LabelImage] = SITEF2019_f1(20,4,3,5);

function [BaseImage,LabelImage] = SITEF2019_f1(u,s,r,t)

elSizeU=reshape(ones(r,1)*(1:s),1,r*s);
BaseImage=[];
LabelImage=[];
for f=1:r*s
    BaseLine=[];
    LabelLine=[];
    LabelCode=rem((1:r*s)+2*f-3,t)+1;
    for g=1:r*s
        ElemN=g+(f-1)*r*s;
        BaseLine=[BaseLine,ElemN*ones(elSizeU(f)*u,elSizeU(g)*u)];
        LabelLine=[LabelLine,LabelCode(g)*ones(elSizeU(f)*u,elSizeU(g)*u)];
    end
    BaseImage=[BaseImage;BaseLine];
    LabelImage=[LabelImage;LabelLine];
end

end