% Function to create Synthetic Images (SITEF 2019)
%  (Base, Label and Reference images)
% 
% created by Andre Marcal,  2 Jan. 2019
%         last modified on  2 Jan. 2019
%
% Citations:
% Marcal, A.R.S. (2019). 
% Segmentation of Sentinel-2 images on SNAP - An evaluation with SITEF. 
% IGARSS 2019 (in press).
% AND / OR
% Marcal, A.R.S., Rodrigues, A.S. (2009). 
% A method for multi-spectral image segmentation evaluation based on synthetic images, 
% Computers & Geosciences, Vol.35, No.8, 1574-1581.
%
% INPUTS
% u - unit size (pixels) 
% s - maximum size (units) 
% r - repetition factor 
% t - No of class types (min 3)
% Train - Training Data (Structure with t elements)
%
% e.g. [BaseIm,LabIm,RefIm] = SITEF2019_f2(8,4,3,5,TrainData);

function [BaseIm,LabIm,RefIm] = SITEF2019_f2(u,s,r,t,Train)

elSizeU=reshape(ones(r,1)*(1:s),1,r*s);
BaseIm=[];
LabIm=[];
RefIm=[];
for f=1:r*s
    BaseLine=[];
    LabelLine=[];
%    RefLine=zeros(0,0,3);
    RefLine=[];
    LabelCode=rem((1:r*s)+2*f-3,t)+1;
    for g=1:r*s
        ElemN=g+(f-1)*r*s;
        BaseLine=[BaseLine,ElemN*ones(elSizeU(f)*u,elSizeU(g)*u)];
        LabelLine=[LabelLine,LabelCode(g)*ones(elSizeU(f)*u,elSizeU(g)*u)];
        LPos=randi(32-elSizeU(f)*u+1);
        CPos=randi(32-elSizeU(g)*u+1);
        RefLine=[RefLine,Train{LabelCode(g)}(LPos:LPos+elSizeU(f)*u-1,CPos:CPos+elSizeU(g)*u-1,:)];
    end
    BaseIm=[BaseIm;BaseLine];
    LabIm=[LabIm;LabelLine];
    RefIm=[RefIm;RefLine];
end

figure, imshow(RefIm)
end